#!/bin/env php
<?php
require_once("/var/www/html/fop2/lib/func.php");
require_once("/var/www/html/fop2/lib/dblib.php");

if(!isset($argv[1])) {
    echo "Usage ".$argv[0]." [file.json]\n";
    exit(1);
};

if(!is_readable($argv[1])) {
    echo $argv[1]." not readable!\n";
    exit(1);
}

$import_file = $argv[1];
$dry = isset($argv[2])?intval($argv[2]):0;

$db = db_connect();

main($db,$import_file);

function parse_amportal_conf($filename) {
    $file = file($filename);
    if (is_array($file)) {
        foreach ($file as $line) {
            if (preg_match("/^\s*([^=]*)\s*=\s*[\"']?([\w\/\:\.\,\}\{\>\<\(\)\*\?\%!=\+\#@&\\$-]*)[\"']?\s*([;].*)?/",$line,$matches)) {
                if(preg_match('/\$amp_conf/',$matches[1])) {
                    $matches[1] = substr($matches[1],11);
                    $matches[1] = trim($matches[1]);
                    $matches[1] = substr($matches[1],0,-2);
                }
                $matches[1] = trim($matches[1]);
                $conf[ $matches[1] ] = trim($matches[2]);
            }
        }
    } else {
        die("<h1>".sprintf("Missing or unreadable config file (%s)...cannot continue", $filename)."</h1>");
    }
    return $conf;
}

function db_connect() {
    if (is_readable("/etc/amportal.conf")) {
        $amp_conf = parse_amportal_conf("/etc/amportal.conf");
        if(!isset($amp_conf['AMPDBENGINE'])) {
            if(is_file("/etc/freepbx/freepbx.conf")) {
                $amp_conf = parse_amportal_conf("/etc/freepbx/freepbx.conf");
            } else if(is_file("/etc/freepbx.conf")) {
                $amp_conf = parse_amportal_conf("/etc/freepbx.conf");
            }
            $DBHOST = $amp_conf['AMPDBHOST'];
            $DBNAME = $amp_conf['AMPDBNAME'];
            $DBUSER = $amp_conf['AMPDBUSER'];
            $DBPASS = $amp_conf['AMPDBPASS'];
        } else {
            $DBHOST = $amp_conf['AMPDBHOST'];
            $DBNAME = $amp_conf['AMPDBNAME'];
            $DBUSER = $amp_conf['AMPDBUSER'];
            $DBPASS = $amp_conf['AMPDBPASS'];
        }

    } else if (is_readable("/etc/freepbx/freepbx.conf")) {
        $amp_conf = parse_amportal_conf("/etc/freepbx/freepbx.conf");
        $DBHOST = $amp_conf['AMPDBHOST'];
        $DBNAME = $amp_conf['AMPDBNAME'];
        $DBUSER = $amp_conf['AMPDBUSER'];
        $DBPASS = $amp_conf['AMPDBPASS'];
    } else if (is_readable("/etc/freepbx.conf")) {
        $amp_conf = parse_amportal_conf("/etc/freepbx.conf");
        $DBHOST = $amp_conf['AMPDBHOST'];
        $DBNAME = $amp_conf['AMPDBNAME'];
        $DBUSER = $amp_conf['AMPDBUSER'];
        $DBPASS = $amp_conf['AMPDBPASS'];
    } else if (is_readable("/etc/asterisk/snep/snep-features.conf")) {
        $DBHOST = 'localhost';
        $DBNAME = 'snep';
        $DBUSER = 'snep';
        $DBPASS = 'sneppass';
        $CDRDBTABLE = "snep.cdr";
    }
    $db = new dbcon($DBHOST, $DBUSER, $DBPASS, $DBNAME, true, true, true);
    return $db;
}

function import_groups($db,$entry) {
    global $dry;
    echo "Importing groups\n";

    $res = $db->consulta("SELECT id,device FROM fop2buttons");
    while($row=$db->fetch_assoc($res)) {
        $all_devices[$row['device']]=$row['id'];
    }

    $res = $db->consulta("SELECT max(id) AS next FROM fop2groups");
    $row = $db->fetch_assoc($res);
    $next_id  = intval($row['next']);
    if($next_id==-1) $next_id=0;

    foreach($entry as $group => $devices) {
       $next_id++;
       $query = "INSERT INTO fop2groups (id,name) VALUES ($next_id,'$group')";

       if($dry==1) {
           echo "$query\n";
       } else {
           $res = $db->consulta($query);
           if(!$res) { print_r($db->error()); }
       }
    
       foreach($devices as $device) {
           if(!array_key_exists($device,$all_devices)) {
                 echo "$device does not exists\n";
           } else {
                 $btn_id = $all_devices[$device];
                 $query = "REPLACE INTO fop2GroupButton (group_name,id_button) VALUES ('$group','$btn_id')";
                 if($dry==1) {
                   echo "$query\n";
                 } else {
                   $db->consulta($query);
                 }
           }
       }
    }
}
function import_permissions($db,$entry) {
    global $dry;
    echo "Importing permissions\n";

    $all_groups = array();
    $res = $db->consulta("SELECT id,name FROM fop2groups");
    while($row=$db->fetch_assoc($res)) {
        $all_groups[$row['name']]=$row['id'];
    }

    $permissions_query = array();
    foreach($entry as $idx => $entry) {

        if(!isset($all_groups[$entry['name_group']])) {
             echo "Group ".$entry['name_group']." not found, ignoring\n";
        } else {
            $permission_name = $entry['name'];
            $permission_permissions = $entry['fop2permissions'];
            $query = "INSERT INTO fop2permissions (name,permissions) VALUES ('$permission_name','$permission_permissions')";
            $permissions_query[] = $query;
            unset($entry['fop2permissions']);
    
            $entry['id_group'] = $all_groups[$entry['name_group']];
            $fields = implode(",",array_keys($entry));
            $values = array_values($entry);
            $count_fields = count($values);
            $placeholders = substr(str_repeat("'%s',",$count_fields),0,-1);

            if($dry==1) {
                $query = "INSERT INTO fop2PermGroup (".$fields.") VALUES ('".implode("','",$values)."')";
                echo "$query\n";
            } else {
                $query = "INSERT INTO fop2PermGroup (".$fields.") VALUES (".$placeholders.")";
                $res = $db->consulta($query,$values);
                if(!$res) {
                    echo $db->error();
                }
            }
        }
    }
    
    foreach(array_unique($permissions_query) as $query) {
        if($dry==1) {
            echo "$query\n";
        } else {
            $res = $db->consulta($query);
            if(!$res) {
                echo $db->error();
            }
        }
    }
}

function import_templates($db,$entry) {
    global $dry;
    echo "Importing templates\n";

    $all_groups = array();
    $res = $db->consulta("SELECT id,name FROM fop2groups");
    while($row=$db->fetch_assoc($res)) {
        $all_groups[$row['name']]=$row['id'];
    }

    foreach($entry as $idx => $entry) {
        $entry_groups = array();
        foreach($entry['groups'] as $group) {
            if(!isset($all_groups[$group])) {
                echo "Group $group is missing, ignoring\n";
            } else {
                $entry_groups[] = $all_groups[$group];
            }
        }
        $entry['groups']=implode(",",$entry_groups);
        $fields = implode(",",array_keys($entry));
        $values = array_values($entry);
        $count_fields = count($values);
        $placeholders = substr(str_repeat("'%s',",$count_fields),0,-1);

        if($dry==1) {
            $query = "INSERT INTO fop2templates (".$fields.") VALUES ('".implode("','",$values)."')";
            echo "$query\n";
        } else {
            $query = "INSERT INTO fop2templates (".$fields.") VALUES (".$placeholders.")";
            $res = $db->consulta($query,$values);
            if(!$res) {
                echo $db->error();
            }
        }
    }
}

function import_users($db,$entry) {
    global $dry;
    echo "Importing users\n";

    $all_groups = array();
    $res = $db->consulta("SELECT id,name FROM fop2groups");
    while($row=$db->fetch_assoc($res)) {
        $all_groups[$row['name']]=$row['id'];
    }

    $all_plugins = array();
    $res = $db->consulta("SELECT id,rawname FROM fop2plugins");
    while($row=$db->fetch_assoc($res)) {
        $all_plugins[]=$row['rawname'];
    }

    $all_templates = array();
    $res = $db->consulta("SELECT id,name FROM fop2templates");
    while($row=$db->fetch_assoc($res)) {
        $all_templates[$row['name']]=$row['id'];
    }

    foreach($entry as $idx => $entry) {

        $groups = explode(",",$entry['groups']);
        $plugins = explode(",",$entry['plugins']);
        $templates = explode(",",$entry['templates']);

        unset($entry['groups']);
        unset($entry['plugins']);
        unset($entry['templates']);

        $exten = $entry['exten'];
        $fields = implode(",",array_keys($entry));
        $values = array_values($entry);
        $count_fields = count($values);
        $placeholders = substr(str_repeat("'%s',",$count_fields),0,-1);

        if($dry==1) {
            $query = "INSERT INTO fop2users (".$fields.") VALUES ('".implode("','",$values)."')";
            echo "$query\n";
            $res=true;
        } else {
            $query = "INSERT INTO fop2users (".$fields.") VALUES (".$placeholders.")";
            $res = $db->consulta($query,$values);
        }

        if(!$res && $dry==0) {
            echo $db->error();
        } else {

            $mygroups=array();
            foreach($groups as $group) {
                if(!array_key_exists($group,$all_groups)) {
                    echo "Warning: group $group does not exist for user $exten\n";
                } else {
                    $mygroups[]=$all_groups[$group];
                }
            }
            foreach($mygroups as $idgroup) {
                if($dry==1) {
                    $query = "INSERT INTO fop2UserGroup (exten,id_group) VALUES ('$exten','$idgroup')";
                    echo "$query\n";
                } else {
                    $query = "INSERT INTO fop2UserGroup (exten,id_group) VALUES ('%s','%s')";
                    $ras = $db->consulta($query,array($exten,$idgroup));
                }
            }

            $myplugins=array();
            foreach($plugins as $plugin) {
                if(!in_array($plugin,$all_plugins)) {
                    echo "Warning: plugin $plugin does not exist for user $exten\n";
                } else {
                    $myplugins[]=$plugin;
                }
            }
            foreach($myplugins as $idplugin) {
                if($dry==1) {
                    $query = "INSERT INTO fop2UserPlugin (exten,id_plugin) VALUES ('$exten','$idplugin')";
                    echo "$query\n";
                } else {
                    $query = "INSERT INTO fop2UserPlugin (exten,id_plugin) VALUES ('%s','%s')";
                    $ras = $db->consulta($query,array($exten,$idplugin));
                }
            }

            $mytemplates=array();
            foreach($templates as $template) {
                if(!array_key_exists($template,$all_templates)) {
                    echo "Warning: template $template does not exist for user $exten\n";
                } else {
                    $mytemplates[]=$all_templates[$template];
                }
            }
            foreach($mytemplates as $idtemplate) {
                if($dry==1) {
                    $query = "INSERT INTO fop2UserTemplate (exten,id_template) VALUES ('$exten','$idtemplate')";
                    echo "$query\n";
                } else {
                    $query = "INSERT INTO fop2UserTemplate (exten,id_template) VALUES ('%s','%s')";
                    $ras = $db->consulta($query,array($exten,$idtemplate));
                }
            }
        }
    }
}

function import_plugins($db,$entry) {
    global $dry;
    echo "Importing plugins\n";
    foreach($entry as $idx => $entry) {
        if($dry==1) {
            $query = "UPDATE fop2plugins SET global = '".$entry['global']."' WHERE rawname='".$entry['rawname']."'";
            echo "$query\n";
        } else {
            $query = "UPDATE fop2plugins SET global = '%d' WHERE rawname='%s'";
            $res = $db->consulta($query,array($entry['global'],$entry['rawname']));
        }
    }
}

function import_buttons($db,$entry) {
    global $dry;
    $res = $db->consulta("SELECT id,device FROM fop2buttons");
    while($row=$db->fetch_assoc($res)) {
        $all_devices[]=$row['device'];
    }

    foreach($entry as $device => $fields) {
        if(!in_array($device,$all_devices)) {
            echo "Skiping button $device, as it is not present\n";
        } else {
            $queryfields=array();
            $queryfieldshidrated=array();
            $queryvals=array();
            echo "Device $device:\n";
            $query = "UPDATE fop2buttons SET ";
            $queryp = "UPDATE fop2buttons SET ";
            foreach($fields as $field=>$val) {
                $queryfields[] = "`$field`='%s'";
                $queryfieldshidrated[] = "`$field`='$val'";
                $queryvals[] = $val;
            }
            $query.= implode(",",$queryfields);
            $queryp.= implode(",",$queryfieldshidrated);
            $query.= " WHERE device='%s'";
            $queryp.= " WHERE device='$device'";
            $queryvals[] = $device;
            if($dry==1) {
                echo "$queryp\n";
            } else {
                $res = $db->consulta($query,$queryvals);
                if(!$res) {
                    echo $db->error();
                }
            }
        }
    }

}

function main($db,$import_file) {

    $json = file_get_contents($import_file);
    $iterable = json_decode($json,1);
    foreach($iterable as $section => $entry) {
        if($section=='groups') {
           import_groups($db,$entry);
        } else if($section=='permissions') {
           import_permissions($db,$entry);
        } else if($section=='templates') {
           import_templates($db,$entry);
        } else if($section=='plugins') {
           import_plugins($db,$entry);
        } else if($section=='buttons') {
           import_buttons($db,$entry);
        } else if($section=='users') {
           import_users($db,$entry);
        }
    }
}





